@@ -1,3 +1,5 @@
+from lmz import Map,Zip,Filter,Grouper,Range,Transpose
+
 from multiprocessing import Pool
 _func = None
 def worker_init(func):
@@ -5,10 +7,11 @@ def worker_init(func):
   _func = func
 def worker(x):
   return _func(x)
-def xmap(func, iterable, processes=None):
-  with Pool(processes, initializer=worker_init, initargs=(func,)) as p:
+def xmap(func, iterable, n_jobs=None, tasksperchild = 1):
+  with Pool(n_jobs, initializer=worker_init, initargs=(func,),maxtasksperchild = tasksperchild) as p:
     return p.map(worker, iterable)
 
+
 import numpy as np
 from scipy.stats import spearmanr
 from scipy import sparse
@@ -50,7 +53,6 @@ def minnum(X):
     return np.nanmin( np.where(np.isinf(X),np.Inf,X) )
 
 
-
 dumpfile = lambda thing, filename: dill.dump(thing, open(filename, "wb"))
 loadfile = lambda filename: dill.load(open(filename, "rb"))
 
@@ -87,6 +89,46 @@ sdumpfile = lambda thing, filename:  sparse.save_npz(filename, thing)
 sloadfile = lambda filename:  sparse.load_npz(filename+'.npz')
 
 
+
+class spacemap():
+    # mapping items to integers...
+    def __init__(self, uniqueitems):
+        self.itemlist = uniqueitems
+        self.integerlist = list(range(len(uniqueitems)))
+        self.len = len(uniqueitems)
+        self.getitem = {i:k for i,k in enumerate(uniqueitems)}
+        self.getint = {k:i for i,k in enumerate(uniqueitems)}
+
+    def encode(self, stuff, fallback = False):
+        return [self.getint.get(x, fallback or x) for x in stuff]
+
+
+    def decode(self, stuff,fallback = False):
+        return [self.getitem.get(x,fallback or -1) for x in stuff]
+
+
+def labelsToIntList(items):
+    sm = spacemap(np.unique(items))
+    return [sm.getint[i] for i in items], sm
+
+def binarize(X,posratio):
+    '''
+    lowest posratio -> 1 ;; rest 0
+    '''
+    argsrt = np.argsort(X)
+    if 0< posratio < 1:
+        cut = max(int(len(X)*(1-posratio)),1)
+    elif len(X) > posratio:
+        cut = len(X) - posratio
+    else:
+        assert False, f'{0<posratio<1= }  {len(X)>posratio= }'
+
+    values = np.ones(len(X), dtype = np.int32)
+    values[argsrt[:cut]] = 0
+    return values
+
+
+
 if __name__ == "__main__":
     a = np.array([0,1,2])
     ndumpfile(a,'adump')
@@ -96,32 +138,6 @@ if __name__ == "__main__":
     sdumpfile(sparse.csr_matrix(a),'sdump')
     print(sloadfile('sdump'))
 
-
-def binarize(X,posratio):
-    '''
-    lowest posratio -> 1 ;; rest 0
-    '''
-    argsrt = np.argsort(X)
-    if 0< posratio < 1:
-        cut = max(int(len(X)*posratio),1)
-    elif len(X) > posratio:
-        cut = -posratio-1
-        print(f"PRENIS")
-    else:
-        assert False ,'we ducked up'
-
-    values = np.zeros(len(X))
-    values[argsrt[:cut]] = 1
-    return values
-
-
-
-
-class spacemap():
-    # mapping items to integers...
-    def __init__(self, items):
-        self.itemlist = items
-        self.integerlist = list(range(len(items)))
-        self.len = len(items)
-        self.getitem = { i:k for i,k in enumerate(items)}
-        self.getint = { k:i for i,k in enumerate(items)}
+    a = np.random.rand(6)
+    print(a)
+    print(binarize(a,2))
